using System;
using System.Drawing;
using System.Collections;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using Microsoft.DirectX.DirectSound;

namespace Asteroids
{
	/// <summary>
	/// Summary description for obstacle.
	/// </summary>
	public class Obstacle : IDisposable
	{
		public Obstacle(Microsoft.DirectX.Direct3D.Device device, float x, float y, float z, Mesh obMesh, Material[] obMaterials, Texture[] obTextures, bool spn)
		{
			//set spin
			spin = spn;

			//store position and set materials and textures
			position = new Vector3(x, y, z);
			obstacleMesh = obMesh;
			obstacleMaterials = obMaterials;
			obstacleTextures = obTextures;
			//make rotation axis and speed random
			rotationspeed = (float)Utility.Rnd.NextDouble() * (float)Math.PI;
			rotationVector = new Vector3((float)Utility.Rnd.NextDouble(), (float)Utility.Rnd.NextDouble(), (float)Utility.Rnd.NextDouble());
			//calculate bounding sphere for Ship
			VertexBuffer vb = obstacleMesh.VertexBuffer;
			try
			{
				//look at entire buffer to calculate
				GraphicsStream stm = vb.Lock(0, 0, LockFlags.None);
				Vector3 center;// required but not used
				radius = Geometry.ComputeBoundingSphere(stm, obstacleMesh.NumberVertices, obstacleMesh.VertexFormat, out center);
				
			}
			finally
			{
				//no matter what unlock and dispose of this vertex buffer
				vb.Unlock();
				vb.Dispose();
			}
		
			
		}
		
		//update obstacles position to match the moving road
		public void Update(float elapsedTime, float speed)
		{
			position.Z += (speed * elapsedTime);
			rotation += (rotationspeed * elapsedTime);
			
		}

		//do the drawing
		public void Draw(Microsoft.DirectX.Direct3D.Device device)
		{
			if(spin == true)
			{
				device.Transform.World = Matrix.RotationAxis(rotationVector, rotation) * Matrix.Translation(position); 
			}
			if(spin == false)//no rotation for ufo mesh
			{
				device.Transform.World =  Matrix.Translation(position); 
			}
			for(int i = 0; i < obstacleMaterials.Length; i++)//draw mesh
			{
				device.Material = obstacleMaterials[i];
				device.SetTexture(0, obstacleTextures[i]);
				obstacleMesh.DrawSubset(i);
			}
			
		}
		
		public float Depth
		{
			get { return position.Z; }
		}

		//collision detection
		public bool IsHittingShip(float shipLocationX, float shipLocationY, float shipLocationZ, float shipDiameter)
		{
			
			float shipXMin = (shipLocationX - 2.5f);
			float shipXMax = (shipLocationX + 2.5f);
			float shipYMin = (shipLocationY - 1); // this kind of forces the bounding sphere into an ellipsoid
			float shipYMax = (shipLocationY + 1); //because ship is much shorter than its largest dimension
			float shipZMin = (shipLocationZ - (shipDiameter / 2));
			float shipZMax = (shipLocationZ + (shipDiameter / 2));

			float obstacleXMin = position.X - radius;
			float obstacleXMax = position.X + radius;
			float obstacleZMin = position.Z - radius;
			float obstacleZMax = position.Z + radius;
			float obstacleYMin = new float();
			float obstacleYMax = new float();
			if(spin)
			{
				obstacleYMin = position.Y - radius;
				obstacleYMax = position.Y + radius;
				
			}
			else
			{
				obstacleYMin = position.Y - 1;
				obstacleYMax = position.Y + 1;
			}

			
			if(obstacleZMax > shipZMin + 4 && obstacleZMin < shipZMax)//a collision is possible adding 4 makes it look better if a wing collides
			{
				//collision from above
				if((obstacleYMin < shipYMax) && (obstacleYMin > shipYMin))
				{
					//collision from left
					if((obstacleXMin < shipXMax) && (obstacleXMin > shipXMin))
					{
						return true;
					}
					//collision from right
					if((obstacleXMax > shipXMin) && (obstacleXMax < shipXMax))
					{
						return true;
					}
				}
				//collision from bottom
				if((obstacleYMax > shipYMin) && (obstacleYMax < shipYMax))
				{
					//collision from left
					if((obstacleXMin < shipXMax) && (obstacleXMin > shipXMin))
					{
						return true;
					}
					//collision from right
					if((obstacleXMax > shipXMin) && (obstacleXMax < shipXMax))
					{
						return true;
					}
				}
				//head on in y axis
				if((obstacleYMax > shipYMax) && (obstacleYMin < shipYMin))
				{
					//collision from left
					if((obstacleXMin < shipXMax) && (obstacleXMin > shipXMin))
					{
						return true;
					}
					//collision from right
					if((obstacleXMax > shipXMin) && (obstacleXMax < shipXMax))
					{
						return true;
					}
				}
			}

			return false;
		
		}
		//play explosion sound fx
		public void PlayExplosion(Microsoft.DirectX.DirectSound.Device soundDevice)
		{
			explosionSound = new SecondaryBuffer(@"..\..\Explosion.wav", soundDevice);
			//explosionSound.Volume = -5;
			
			explosionSound.Play(0, BufferPlayFlags.Default);
						
		}
		//public obstacle members
		public float LocationX
		{
			get{ return position.X; }
			
		}
		public float LocationY
		{
			get{ return position.Y; }
			
		}
		public float LocationZ
		{
			get{ return position.Z; }
		}
		public float Radius
		{
			get{ return radius; }
		}

		//object constants
		private const int NumberMeshTypes = 5;//sphere, cube, torus, cylinder, teapot
		private const float ObjectLength = 3.0f;
		private const float ObjectRadius = ObjectLength / 2.0f;
		private const int ObjectStacksSlices = 18;

		

	
		//mesh info
		private Mesh obstacleMesh = null;
		private Material[] obstacleMaterials = null;
		private Texture[] obstacleTextures = null;

		//private Mesh obstacleMesh = null;
		//private Material obstacleMaterial;
		private Vector3 position;
		
		//private bool isTeapot;
		private float radius;
		//to control rotation
		private float rotation = 0.0f;
		private float rotationspeed = 0.0f;
		private Vector3 rotationVector;
		
		//for explosion
		
		private int lastIndexTick = System.Environment.TickCount;
		private SecondaryBuffer explosionSound = null;

		//to control spin
		private bool spin;

		
		
		#region IDisposable Members

		public void Dispose()
		{
			// TODO:  Add Obstacle.Dispose implementation
		}

		#endregion
	}
	public class Obstacles : IEnumerable //to add and remove obstacles from game
	{
		/// <summary>
		/// Indexer for this class for direct access to ann obstacle
		/// </summary>
		
		public Obstacle this[int index]
		{
			get
			{
				return (Obstacle)obstacleList[index];
			}
		}

		//get enumerator from array list
		public IEnumerator GetEnumerator()//so we can use foreach
		{
			return obstacleList.GetEnumerator();
		}

		/// <summary>
		/// add obstacle to list
		/// 
		/// </summary>
		/// <paramref name=" obstacle"> the obstacle to add</paramref>
		public void Add(Obstacle obstacle)
		{
			obstacleList.Add(obstacle);
		}

		/// <summary>
		/// remove obstacle from list
		/// </summary>
		/// <paramref name=" obstacle">the obstacle to remove</paramref>
		public void Remove(Obstacle obstacle)
		{
			obstacleList.Remove(obstacle);
		}

		/// <summary>
		/// clear obstacle list
		/// </summary>
		public void Clear()
		{
			obstacleList.Clear();
		}
	
		private ArrayList obstacleList = new ArrayList();
	}


}
